<?php

namespace Drupal\commerce_stripe_payment_request_button\Controller;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\Payment;
use Drupal\commerce_shipping\Entity\Shipment;
use Drupal\commerce_shipping\ShipmentItem;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;
use Drupal\physical\Weight;
use Drupal\physical\WeightUnit;
use Drupal\profile\Entity\Profile;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class PaymentRequestButtonCheckout {

  public function handleOrder(Request $request) {
    // Transform JSON values to array
    $values = Json::decode($request->getContent());
    $order_id = $values["data"]["order_id"];
    $shipping_info = $values["data"]["result"]["shippingAddress"];

    $carts = \Drupal::getContainer()
      ->get('commerce_cart.cart_provider')
      ->getCarts();
    $active_cart = array_filter($carts, function (OrderInterface $cart) use ($order_id) {
      return $cart->id() == $order_id;
    });
    $order = reset($active_cart);


    // Init Stripe and config
    $config = \Drupal::config('commerce_payment.commerce_payment_gateway.commerce_stripe_payment_request_button')
      ->get('configuration');
    \Stripe\Stripe::setApiKey($config['secret_key']);
    $description = $values["data"]["result"]["token"]["card"]["id"] . ', ' . \Drupal::config('system.site')
        ->get('name');

    if (empty($order) || $order->getState()->value == 'canceled') {
      $response = [
        'redirect_uri' => Url::fromRoute(Url::fromRoute('commerce_cart.page')
          ->toString()),
      ];
      return new JsonResponse($response);
    }

    try {
      // Charge the user's card immediately.
      $charge = \Stripe\Charge::create([
        'amount' => $values["data"]["payment_request"]["total"]["amount"],
        'currency' => $values["data"]["payment_request"]["currency"],
        'description' => $description,
        'metadata' => [
          "order_id" => $values["data"]["order_id"],
          "email" => $values["data"]["result"]["payerEmail"],
        ],
        'source' => $values["data"]["token"],
      ]);
    } catch (\Stripe\Error\Base $e) {
      watchdog_exception('commerce_stripe_payment_request', $e);
      // Server will return 500 to the frontend.
      \Drupal::messenger()->addError($e->getMessage());
    }

    if ($order->hasItems()) {
      // Put order in proper status and assign values
      $order->get('checkout_flow')->setValue(NULL);
      $order->get('checkout_step')->setValue("payment");
      $order->setEmail($values["data"]["result"]["payerEmail"]);
      $order->get('payment_gateway')
        ->setValue("commerce_stripe_payment_request_button");
      $order->save();

      // Create payment for this order and give it values
      $payment_gateway = \Drupal::entityTypeManager()
        ->getStorage('commerce_payment_gateway')
        ->load('commerce_stripe_payment_request_button');

      $current_user = $order->getCustomerId();

      $payment = Payment::create([
        'state' => 'new',
        'amount' => $order->getTotalPrice(),
        'payment_gateway' => $payment_gateway->id(),
        'order_id' => $order->id(),
        'remote_id' => $charge->id,
        'payment_gateway_mode' => $payment_gateway->getPlugin()->getMode(),
        'expires' => 0,
        'uid' => $current_user,
      ]);
      $payment->save();

      // Deal with billing profile
      $profile = $order->getBillingProfile();

      if (empty($profile)) {
        $profile = Profile::create([
          'type' => 'customer',
          'uid' => $current_user,
        ]);
        $profile->save();
      }

      $current_name = isset($values["data"]["result"]["payerName"]) ?: $values["data"]["result"]["token"]["card"]["name"];
      $names = explode(' ', $current_name);
      $given_name = array_shift($names);
      $family_name = implode(' ', $names);

      $profile->address->given_name = $given_name;
      $profile->address->family_name = $family_name;
      $profile->address->country_code = $values["data"]["result"]["token"]["card"]["country"];
      $profile->address->locality = $values["data"]["result"]["token"]["card"]["address_city"];
      $profile->address->administrative_area = $values["data"]["result"]["token"]["card"]["address_state"];
      $profile->address->postal_code = $values["data"]["result"]["token"]["card"]["address_zip"];
      $profile->address->address_line1 = $values["data"]["result"]["token"]["card"]["address_line1"];
      $profile->address->address_line2 = $values["data"]["result"]["token"]["card"]["address_line2"];
      $profile->save();

      $order->setBillingProfile($profile);
      $order->save();

      // Deal with shipping stuff
      if ($order->get('shipments')->count() == 0) {

        $first_shipment = Shipment::create([
          'type' => 'default',
          'order_id' => $order_id,
          'title' => 'Shipment',
          'state' => 'ready',
        ]);

        // Loop through order items and add them to shipment
        foreach ($order->getItems() as $order_item) {
          $quantity = $order_item->getQuantity();
          $purchased_entity = $order_item->getPurchasedEntity();

          if ($purchased_entity->get('weight')->isEmpty()) {
            $weight = new Weight(1, WeightUnit::GRAM);
          }
          else {
            $weight_item = $purchased_entity->get('weight')->first();
            $weight = $weight_item->toMeasurement();
          }

          $shipment_item = new ShipmentItem([
            'order_item_id' => $order_item->id(),
            'title' => $purchased_entity->label(),
            'quantity' => $quantity,
            'weight' => $weight->multiply($quantity),
            'declared_value' => $order_item->getTotalPrice(),
          ]);
          $first_shipment->addItem($shipment_item);
        }

        // Find rate, we use the first one, pretending there is only one we want to use, fixed rate for order
        $shipping_method_storage = \Drupal::entityTypeManager()
          ->getStorage('commerce_shipping_method');
        $shipping_method = $shipping_method_storage->load($values["data"]["result"]["shippingOption"]["id"]);

        // Add proper shipping method
        $first_shipment->setShippingMethod($shipping_method);
        $shipping_method_plugin = $shipping_method->getPlugin();
        $shipping_rates = $shipping_method_plugin->calculateRates($first_shipment);
        $shipping_rate = reset($shipping_rates);
        $shipping_service = $shipping_rate->getService()->getId();
        $shipping_method_amount = $shipping_rate->getAmount();
        $first_shipment->setAmount($shipping_method_amount);

        // We only get the full name from Apple/Google pay, so we need to "guess" the given
        // name and the family name.
        $names = explode(' ', $shipping_info['recipient']);
        $given_name = array_shift($names);
        $family_name = implode(' ', $names);

        $profile = \Drupal::entityTypeManager()->getStorage('profile')->create([
          'uid' => $order->getCustomerId(),
          'type' => 'customer',
        ]);
        $profile->address->given_name = $given_name;
        $profile->address->family_name = $family_name;

        $profile->address->given_name = $given_name;
        $profile->address->family_name = $family_name;
        $profile->address->country_code = $shipping_info['country'];
        $profile->address->locality = $shipping_info['city'];
        $profile->address->administrative_area = $shipping_info['dependentLocality'];
        $profile->address->postal_code = $shipping_info['postalCode'];
        $profile->address->address_line1 = $shipping_info["addressLine"][0];
        $profile->address->address_line2 = isset($shipping_info["addressLine"][1]) ?: "";
        $profile->save();

        $first_shipment->setShippingProfile($profile);
        $first_shipment->setShippingService($shipping_service);
        $first_shipment->save();
        $order->set('shipments', $first_shipment);
        $order->save();
      }

    }

    // Gets url from which payment step is set and sends it to JS
    $response = [
      'redirect_uri' => Url::fromRoute('commerce_checkout.form', [
        'commerce_order' => $order->id(),
      ])->toString(),
    ];


    return new JsonResponse($response);
  }


}
